@@ -4,11 +4,11 @@
  *    http://www.robobunny.com/projects/asciiquarium/
  *
  * Ported to KDE by Maksim Orlovich <maksim@kde.org> and
- * Michael Pyne <michael.pyne@kdemail.net>.
+ * Michael Pyne <mpyne@kde.org>.
  *
  * Copyright (c) 2003 Kirk Baucom     <kbaucom@schizoid.com>
  * Copyright (c) 2005 Maksim Orlovich <maksim@kde.org>
- * Copyright (c) 2005, 2008 Michael Pyne <michael.pyne@kdemail.net>
+ * Copyright (c) 2005, 2008 Michael Pyne <mpyne@kde.org>
  *
  * ASCII Art was mostly created by Joan Stark:
  *    http://www.geocities.com/SoHo/7373/
@@ -76,6 +76,7 @@ AASaver::AASaver(WId id): KScreenSaver(id)
 
     setAttribute(Qt::WA_OpaquePaintEvent);
     setAttribute(Qt::WA_NoSystemBackground);
+    setAttribute(Qt::WA_PaintOnScreen);
     update(rect());
 }
 
@@ -92,8 +93,8 @@ QString AASaver::randColor(QString color_mask)
     char colors[] = {'c','C','r','R','y','Y','b','B','g','G','m','M'};
     for (int i = 1; i <= 9; ++i)
     {
-        char color = colors[m_randomSequence.getLong(ARRAY_SIZE(colors))];
-        color_mask.replace('0' + i, color);
+        const char color = colors[m_randomSequence.getLong(ARRAY_SIZE(colors))];
+        color_mask.replace(QLatin1Char('0' + i), QLatin1Char(color));
     }
     return color_mask;
 }
@@ -105,7 +106,7 @@ double AASaver::doubleRand(int max)
 
 void AASaver::addCastle()
 {
-    QString castle_image =
+    QString castle_image = QLatin1String(
             "               T~~\n"
             "               |\n"
             "              /^\\\n"
@@ -118,10 +119,10 @@ void AASaver::addCastle()
             " | =_    |= - ___    | =_ =  |\n"
             " |=  []- |-  /| |\\   |=_ =[] |\n"
             " |- =_   | =| | | |  |- = -  |\n"
-            " |_______|__|_|_|_|__|_______|\n";
+            " |_______|__|_|_|_|__|_______|\n" );
 
 
-    QString castle_mask =
+    QString castle_mask = QLatin1String(
         "                RR\n"
         "\n"
         "              yyy\n"
@@ -134,7 +135,7 @@ void AASaver::addCastle()
         "              yyy\n"
         "             yy yy\n"
         "            y y y y\n"
-        "            yyyyyyy\n";
+        "            yyyyyyy\n" );
 
     Frame f(castle_image, castle_mask, 0x686868/* XXX: why grey? */ );
 
@@ -147,10 +148,10 @@ void AASaver::addCastle()
 void AASaver::addEnvironment()
 {
     QString water_line_segment[] = {
-        "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~",
-        "^^^^ ^^^  ^^^   ^^^    ^^^^      ",
-        "^^^^      ^^^^     ^^^    ^^     ",
-        "^^      ^^^^      ^^^    ^^^^^^  "
+        QLatin1String( "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~" ),
+        QLatin1String( "^^^^ ^^^  ^^^   ^^^    ^^^^      " ),
+        QLatin1String( "^^^^      ^^^^     ^^^    ^^     " ),
+        QLatin1String( "^^      ^^^^      ^^^    ^^^^^^  " )
     };
 
     // tile the segments so they stretch across the screen
@@ -241,14 +242,14 @@ QList<int> Seaweed::m_positions;
 
 void AASaver::addSeaweed(Screen* screen)
 {
-    QString seaweed_image[] = {"", ""};
+    QString seaweed_image[] = {QLatin1String( "" ), QLatin1String( "" )};
     int height = m_randomSequence.getLong(5) + 3;
     for (int i = 1; i <= height; ++i)
     {
         int left_side  = i % 2;
         int right_side = !left_side;
-        seaweed_image[left_side]  += "(\n";
-        seaweed_image[right_side] += " )\n";
+        seaweed_image[left_side]  += QLatin1String( "(\n" );
+        seaweed_image[right_side] += QLatin1String( " )\n" );
     }
 
     int x = m_randomSequence.getLong(screen->width() - 2) + 1;
@@ -288,9 +289,9 @@ public:
     AirBubble(Screen *screen, int x, int y, int z) :
         Sprite(screen, x, y, z), m_startY(y)
     {
-        addFrame(Frame(".", QString(), 0x18B2B2));
-        addFrame(Frame("o", QString(), 0x18B2B2));
-        addFrame(Frame("O", QString(), 0x18B2B2));
+        addFrame(Frame(QLatin1String( "." ), QString(), 0x18B2B2));
+        addFrame(Frame(QLatin1String( "o" ), QString(), 0x18B2B2));
+        addFrame(Frame(QLatin1String( "O" ), QString(), 0x18B2B2));
 
         setFrameDelay(100);
     }
@@ -514,183 +515,208 @@ void AASaver::addAllFish()
 Sprite *AASaver::newFish(Screen *screen)
 {
     QString fish_image[] = {
-"       \\\n"
-"     ...\\..,\n"
-"\\" "{}" "/'       \\\n" // trigraphs suck
-" >=     (  ' >\n"
-"/{}\\      / /\n"
-"    `\"'\"'/''\n",
+QLatin1String( "       \\\n"
+               "     ...\\..,\n"
+               "\\" "{}" "/'       \\\n" // trigraphs suck
+               " >=     (  ' >\n"
+               "/{}\\      / /\n"
+               "    `\"'\"'/''\n" ),
 
-"       2\n"
-"     1112111\n"
-"6  11       1\n"
-" 66     7  4 5\n"
-"6  1      3 1\n"
-"    11111311\n",
+QLatin1String( "       2\n"
+               "     1112111\n"
+               "6  11       1\n"
+               " 66     7  4 5\n"
+               "6  1      3 1\n"
+               "    11111311\n" ),
 
 //////////////////////////////
-"      /\n"
-"  ,../...\n"
-" /       '\\" "{}" "/\n" // trigraphs suck
-"< '  )     =<\n"
-" \\ \\      /{}\\\n"
-"  `'\\'\"'\"'\n",
+QLatin1String( "      /\n"
+               "  ,../...\n"
+               " /       '\\" "{}" "/\n" // trigraphs suck
+               "< '  )     =<\n"
+               " \\ \\      /{}\\\n"
+               "  `'\\'\"'\"'\n" ),
 
-"      2\n"
-"  1112111\n"
-" 1       11  6\n"
-"5 4  7     66\n"
-" 1 3      1  6\n"
-"  11311111\n",
+QLatin1String( "      2\n"
+               "  1112111\n"
+               " 1       11  6\n"
+               "5 4  7     66\n"
+               " 1 3      1  6\n"
+               "  11311111\n" ),
 //////////////////////////////
-"    \\\n"
-"\\?/--\\\n"
-">=  (o>\n"
-"/?\\__/\n"
-"    /\n",
+QLatin1String( "    \\\n"
+               "\\?/--\\\n"
+               ">=  (o>\n"
+               "/?\\__/\n"
+               "    /\n" ),
 
-"    2\n"
-"6 1111\n"
-"66  745\n"
-"6 1111\n"
-"    3\n",
+QLatin1String( "    2\n"
+               "6 1111\n"
+               "66  745\n"
+               "6 1111\n"
+               "    3\n" ),
 
 //////////////////////////////
-"  /\n"
-" /--\\?/\n"
-"<o)  =<\n"
-" \\__/?\\\n"
-"  \\\n",
+QLatin1String( "  /\n"
+               " /--\\?/\n"
+               "<o)  =<\n"
+               " \\__/?\\\n"
+               "  \\\n" ),
 
-"  2\n"
-" 1111 6\n"
-"547  66\n"
-" 1111 6\n"
-"  3\n",
+QLatin1String( "  2\n"
+               " 1111 6\n"
+               "547  66\n"
+               " 1111 6\n"
+               "  3\n" ),
 
 //////////////////////////////
+QLatin1String(
 "       \\:.\n"
 "\\;,{}?,;\\\\\\\\\\,,\n"
 "  \\\\\\\\\\;;:::::o\n"
 "  ///;;::::::::<\n"
-" /;`?``/////``\n",
+" /;`?``/////``\n" ),
 
+QLatin1String(
 "       222\n"
 "666   1122211\n"
 "  6661111111114\n"
 "  66611111111115\n"
-" 666 113333311\n",
+" 666 113333311\n" ),
 
 //////////////////////////////
+QLatin1String(
 "      .:/\n"
 "   ,,///;,{}?,;/\n"
 " o:::::::;;///\n"
 ">::::::::;;\\\\\\\n"
-"  ''\\\\\\\\\\''?';\\\n",
+"  ''\\\\\\\\\\''?';\\\n" ),
 
+QLatin1String(
 "      222\n"
 "   1122211   666\n"
 " 4111111111666\n"
 "51111111111666\n"
-"  113333311 666\n",
+"  113333311 666\n" ),
 
 //////////////////////////////
+QLatin1String(
 "  __\n"
 "><_'>\n"
-"   '\n",
+"   '\n" ),
 
+QLatin1String(
 "  11\n"
 "61145\n"
-"   3\n",
+"   3\n" ),
 
 //////////////////////////////
+QLatin1String(
 " __\n"
 "<'_><\n"
-" `\n",
+" `\n" ),
 
+QLatin1String(
 " 11\n"
 "54116\n"
-" 3\n",
+" 3\n" ),
 
 //////////////////////////////
+QLatin1String(
 "   ..\\,\n"
 ">='   ('>\n"
-"  '''/''\n",
+"  '''/''\n" ),
 
+QLatin1String(
 "   1121\n"
 "661   745\n"
-"  111311\n",
+"  111311\n" ),
 
 //////////////////////////////
+QLatin1String(
 "  ,/..\n"
 "<')   `=<\n"
-" ``\\```\n",
+" ``\\```\n" ),
 
+QLatin1String(
 "  1211\n"
 "547   166\n"
-" 113111\n",
+" 113111\n" ),
 
 //////////////////////////////
+QLatin1String(
 "   \\\n"
 "  / \\\n"
 ">=_('>\n"
 "  \\_/\n"
-"   /\n",
+"   /\n" ),
 
+QLatin1String(
 "   2\n"
 "  1 1\n"
 "661745\n"
 "  111\n"
-"   3\n",
+"   3\n" ),
 
 //////////////////////////////
+QLatin1String(
 "  /\n"
 " / \\\n"
 "<')_=<\n"
 " \\_/\n"
-"  \\\n",
+"  \\\n" ),
 
+QLatin1String(
 "  2\n"
 " 1 1\n"
 "547166\n"
 " 111\n"
-"  3\n",
+"  3\n" ),
 
 //////////////////////////////
+QLatin1String(
 "  ,\\\n"
 ">=('>\n"
-"  '/\n",
+"  '/\n" ),
 
+QLatin1String(
 "  12\n"
 "66745\n"
-"  13\n",
+"  13\n" ),
 
 //////////////////////////////
+QLatin1String(
 " /,\n"
 "<')=<\n"
-" \\`\n",
+" \\`\n" ),
 
+QLatin1String(
 " 21\n"
 "54766\n"
-" 31\n",
+" 31\n" ),
 
 //////////////////////////////
+QLatin1String(
 "  __\n"
 "\\/ o\\\n"
-"/\\__/\n",
+"/\\__/\n" ),
 
+QLatin1String(
 "  11\n"
 "61 41\n"
-"61111\n",
+"61111\n" ),
 
 //////////////////////////////
+QLatin1String(
 " __\n"
 "/o \\/\n"
-"\\__/\\\n",
+"\\__/\\\n" ),
 
+QLatin1String(
 " 11\n"
 "14 16\n"
-"11116\n"
+"11116\n" )
+
 };
 
     // # 1: body
@@ -707,7 +733,7 @@ Sprite *AASaver::newFish(Screen *screen)
     int depth = 3 + m_randomSequence.getLong(18);
 
     QString color_mask = fish_image[fish_index+1];
-    color_mask.replace('4', 'W');
+    color_mask.replace(QLatin1Char( '4' ), QLatin1Char( 'W' ));
 
     color_mask = randColor(color_mask);
 
@@ -756,34 +782,37 @@ public:
         Sprite(screen, 0, 0, depth, 450 /* frame Delay */)
     {
         QString splats[] = {
+QLatin1String(
 "\n"
 "   .\n"
 "  ***\n"
 "   '\n"
-""
+"" )
 ,
-
+QLatin1String(
 "\n"
 " \",*;`\n"
 " \"*,**\n"
 " *\"'~'\n"
-""
+"" )
 ,
+QLatin1String(
 "  , ,\n"
 " \" \",\"'\n"
 " *\" *'\"\n"
 "  \" ; .\n"
-""
+"" )
 ,
+QLatin1String(
 "* ' , ' `\n"
 "' ` * . '\n"
 " ' `' \",'\n"
 "* ' \" * .\n"
-"\" * ', '"
+"\" * ', '" )
         };
 
         for(unsigned i = 0; i < ARRAY_SIZE(splats); ++i)
-            addFrame(Frame(splats[i], QString(), 0xB21818, ' '));
+            addFrame(Frame(splats[i], QString(), 0xB21818, QLatin1Char( ' ' )));
 
         QRect r(center, QSize(9, 5));
         r.moveCenter(center);
@@ -816,7 +845,7 @@ public:
         if(m_direct > 0) // Moving to right.
             m_realX = -10;
 
-        addFrame(Frame("{}{}{}{}", QString(), 0));
+        addFrame(Frame(QLatin1String(  "{}{}{}{}" ), QString(), 0));
     }
 
     /// Returns true since we can collide.
@@ -843,6 +872,7 @@ public:
 void AASaver::addShark(Screen* screen)
 {
     QString shark_image[] = {
+QLatin1String(
 "                              __\n"
 "                             ( `\\\n"
 "  ,{}{}{}{}{}{}{}{}{}{}{}{}{}" ")   `\\\n" // trigraphs suck
@@ -852,8 +882,8 @@ void AASaver::addShark(Screen* screen)
 "    >                     _.-'      .((      ._     )\n"
 "  .`.-`--...__         .-'     -.___.....-(|/|/|/|/'\n"
 " ;.'{}{}{}{}?`. ...----`.___.',,,_______......---'\n"
-" '{}{}{}{}{}?" "'-'\n", // trigraphs suck
-
+" '{}{}{}{}{}?" "'-'\n" ), // trigraphs suck
+QLatin1String(
 "                                                     \n"
 "                                                     \n"
 "                                                     \n"
@@ -863,7 +893,8 @@ void AASaver::addShark(Screen* screen)
 "                                                     \n"
 "                                          cWWWWWWWW  \n"
 "                                                     \n"
-"                                                     \n",
+"                                                     \n" ),
+QLatin1String(
 
 "                     __\n"
 "                    /' )\n"
@@ -874,8 +905,8 @@ void AASaver::addShark(Screen* screen)
 "(     _.      )).      `-._                     <\n"
 " `\\|\\|\\|\\|)-.....___.-     `-.         __...--'-.'.\n"
 "   `---......_______,,,`.___.'----... .'{}{}{}{}?`.;\n"
-"                                     `-`{}{}{}{}{}?`\n",
-
+"                                     `-`{}{}{}{}{}?`\n" ),
+QLatin1String(
 "                                                     \n"
 "                                                     \n"
 "                                                     \n"
@@ -885,7 +916,7 @@ void AASaver::addShark(Screen* screen)
 "                                                     \n"
 "  WWWWWWWWc                                          \n"
 "                                                     \n"
-"                                                     \n"
+"                                                     \n" )
     };
 
     int shark_num   = m_randomSequence.getLong(ARRAY_SIZE(shark_image)/2);
@@ -908,6 +939,76 @@ void AASaver::addShark(Screen* screen)
     screen->addSprite(teeth);
 }
 
+void AASaver::addSubmarine(Screen* screen)
+{
+    QString submarine_image[] = {
+QLatin1String(
+"{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}/|]\n"
+"{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}||\n"
+"{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}?_||__\n"
+"{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}/     |\n"
+"{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}?/      |\n"
+"{}{}{}{}{}{}{}{}{}________________/_______|_______________________\n"
+"?_{}?____________/ ============================================== >\n"
+"(_)?/                                                            /\n"
+"?(=<                                       O       O       O    /\n"
+"(_)?\\__________________________________________________________/\n"),
+
+QLatin1String(
+"                                                                   \n"
+"                                                                   \n"
+"                                                                   \n"
+"                                                                   \n"
+"                                                                   \n"
+"                                                                   \n"
+"                   cccccccccccccccccccccccccccccccccccccccccccccc  \n"
+"                                                                   \n"
+"                                           w       w       w       \n"
+"                                                                   \n"),
+
+QLatin1String(
+"                          [|\\{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}\n"
+"                           ||{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}\n"
+"                         __||_{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}?\n"
+"                        |     \\{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}\n"
+"                        |      \\{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}?\n"
+" _______________________|_______\\________________{}{}{}{}{}{}{}{}{}\n"
+"< ============================================== \\____________{}?_\n"
+" \\                                                            \\?(_)\n"
+"  \\    O       O       O                                       >=)\n"
+"   \\__________________________________________________________/?(_)\n"),
+
+QLatin1String(
+"                                                                   \n"
+"                                                                   \n"
+"                                                                   \n"
+"                                                                   \n"
+"                                                                   \n"
+"                                                                   \n"
+"  cccccccccccccccccccccccccccccccccccccccccccccc                   \n"
+"                                                                   \n"
+"       w       w       w                                           \n"
+"                                                                   \n")
+    };
+
+    int submarine_num = m_randomSequence.getLong(17) % 2;
+    QString mask = submarine_image[2 * submarine_num + 1];
+
+    int x = -69, dir = 1;
+
+    if(submarine_num == 1) {
+        x = screen->width() - 2;
+        dir = -1;
+    }
+
+    int y = 9 + m_randomSequence.getLong(screen->height() - (10 + 9));
+    int depth = 3 + m_randomSequence.getLong(18);
+
+    RandomMovingSprite* submarine = new RandomMovingSprite(screen, dir, 2, x, y, depth);
+    submarine->addFrame(Frame(submarine_image[2 * submarine_num], mask, 0xFFFF00));
+    screen->addSprite(submarine);
+}
+
 void AASaver::addBubble(Screen *screen, int x, int y, int z)
 {
     screen->addSprite(new AirBubble(screen, x, y, z));
@@ -916,33 +1017,34 @@ void AASaver::addBubble(Screen *screen, int x, int y, int z)
 void AASaver::addShip(Screen* screen)
 {
     QString ship_image[] = {
+QLatin1String(
 "     |    |    |\n"
 "    )_)  )_)  )_)\n"
 "   )___))___))___)\\\n"
 "  )____)____)_____)\\\\\n"
 "_____|____|____|____\\\\\\__\n"
-"\\                   /",
-
+"\\                   /" ),
+QLatin1String(
 "     y    y    y\n"
 "                 \n"
 "                  w\n"
 "                   ww\n"
 "yyyyyyyyyyyyyyyyyyyywwwyy\n"
-"y                   y",
-
+"y                   y" ),
+QLatin1String(
 "         |    |    |\n"
 "        (_(  (_(  (_(\n"
 "      /(___((___((___(\n"
 "    //(_____(____(____(\n"
 "__///____|____|____|_____\n"
-"    \\                   /",
-
+"    \\                   /" ),
+QLatin1String(
 "         y    y    y\n"
 "                 \n"
 "      w            \n"
 "    ww               \n"
 "yywwwyyyyyyyyyyyyyyyyyyyy\n"
-"    y                   y"
+"    y                   y" )
     };
 
     int ship_num = m_randomSequence.getLong(17) % 2; // right == 0, left == 1
@@ -961,61 +1063,63 @@ void AASaver::addShip(Screen* screen)
 void AASaver::addWhale(Screen* screen)
 {
     QString whale_image[] = {
+        QLatin1String(
 "        .-----:\n"
 "      .'       `.\n"
 ",{}{}/       (o) \\\n"
-"\\`._/          ,__)",
-
+"\\`._/          ,__)" ),
+QLatin1String(
 "             C C\n"
 "           CCCCCCC\n"
 "           C  C  C\n"
 "        BBBBBBB\n"
 "      BB       BB\n"
 "B    B       BWB B\n"
-"BBBBB          BBBB",
-
+"BBBBB          BBBB" ),
+QLatin1String(
 "    :-----.\n"
 "  .'       `.\n"
 " / (o)       \\{}{},\n"
-"(__,          \\_.'/",
-
+"(__,          \\_.'/" ),
+QLatin1String(
 "   C C\n"
 " CCCCCCC\n"
 " C  C  C\n"
 "    BBBBBBB\n"
 "  BB       BB\n"
 " B BWB       B    B\n"
-"BBBB          BBBBB"
+"BBBB          BBBBB" )
     };
 
     QString spouty[] = {
+        QLatin1String(
 "\n"
 "\n"
-"   :",
-
+"   :" ),
+QLatin1String(
 "\n"
 "   :\n"
-"   :",
-
+"   :" ),
+QLatin1String(
 "  . .\n"
 "  -:-\n"
-"   :",
-
+"   :" ),
+QLatin1String(
 "  . .\n"
 " .-:-.\n"
-"   :",
-
+"   :" ),
+QLatin1String(
 "  . .\n"
 "'.-:-.`\n"
-"'  :  '",
-
+"'  :  '" ),
+QLatin1String(
 "\n"
 " .- -.\n"
-";  :  ;",
-
+";  :  ;" ),
+QLatin1String(
 "\n"
 "\n"
-";     ;"
+";     ;" )
     };
 
     int whale_num = m_randomSequence.getLong(2); // 0 = right, 1 = left
@@ -1035,7 +1139,7 @@ void AASaver::addWhale(Screen* screen)
     whale->setFrameTime(40);
 
     // We have to add some frames now.  The first five will have no water spout.
-    QString blankWhaleFrame = QString("\n\n\n") + whale_image[2 * whale_num];
+    QString blankWhaleFrame = QLatin1String("\n\n\n") + whale_image[2 * whale_num];
 
     for(unsigned i = 0; i < 5; ++i)
         whale->addFrame(Frame(blankWhaleFrame, mask, 0xFFFFFF));
@@ -1044,17 +1148,17 @@ void AASaver::addWhale(Screen* screen)
     QString whaleFrame = whale_image[2 * whale_num];
     for (unsigned i = 0; i < ARRAY_SIZE(spouty); ++i)
     {
-        QStringList spoutLines = spouty[i].split('\n');
+        QStringList spoutLines = spouty[i].split(QLatin1Char('\n'));
         QString spout;
         QString padding;
 
-        padding.fill(' ', spout_align);
+        padding.fill(QLatin1Char( ' ' ), spout_align);
 
         // Move spout over an appropriate distance to line up right.
         foreach(const QString &spoutLine, spoutLines) {
             spout += padding;
             spout += spoutLine;
-            spout += '\n';
+            spout += QLatin1Char('\n');
         }
 
         // Add spout to whale frame.
@@ -1067,6 +1171,7 @@ void AASaver::addWhale(Screen* screen)
 void AASaver::addBigFish(Screen* screen)
 {
     QString big_fish_image[] = {
+QLatin1String(
 " ______\n"
 "`\"\"-.  `````-----.....__\n"
 "     `.  .      .       `-.\n"
@@ -1080,8 +1185,8 @@ void AASaver::addBigFish(Screen* screen)
 " '{}?.'  .    .     .   .-'\n"
 "   .'____....----''.'=.'\n"
 "   \"\"{}{}{}{}{}{}?.'.'\n"
-"               ''\"'`",
-
+"               ''\"'`" ),
+QLatin1String(
 " 111111\n"
 "11111  11111111111111111\n"
 "     11  2      2       111\n"
@@ -1095,8 +1200,8 @@ void AASaver::addBigFish(Screen* screen)
 " 1   11  2    2     2   111\n"
 "   111111111111111111111\n"
 "   11             1111\n"
-"               11111",
-
+"               11111" ),
+QLatin1String(
 "                           ______\n"
 "          __.....-----'''''  .-\"\"'\n"
 "       .-'       .      .  .'\n"
@@ -1110,8 +1215,8 @@ void AASaver::addBigFish(Screen* screen)
 "       `-.   .     .    .  `.{}?`\n"
 "          `.=`.``----....____`.\n"
 "            `.`.{}{}{}{}{}{}?\"\"\n"
-"              '`\"``",
-
+"              '`\"``" ),
+QLatin1String(
 "                           111111\n"
 "          11111111111111111  11111\n"
 "       111       2      2  11\n"
@@ -1125,7 +1230,7 @@ void AASaver::addBigFish(Screen* screen)
 "       111   2     2    2  11   1\n"
 "          111111111111111111111\n"
 "            1111             11\n"
-"              11111"
+"              11111" )
     };
 
     int big_fish_num = m_randomSequence.getLong(2); // right = 0, left = 1
@@ -1150,69 +1255,71 @@ void AASaver::addBigFish(Screen* screen)
 void AASaver::addNessie(Screen* screen)
 {
     QString nessie_image[] = {
+QLatin1String(
 "                                                          ____\n"
 "            __{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}/   o  \\\n"
 "          /    \\{}{}{}{}_{}{}{}{}{}{}{}{}{}{}?_{}{}{}?/     ____ >\n"
 "  _{}{}{}|  __  |{}{}?/   \\{}{}{}{}_{}{}{}{}/   \\{}{}|     |\n"
-" | \\{}{}?|  ||  |{}{}|     |{}{}?/   \\{}{}?|     |{}?|     |",
-
+" | \\{}{}?|  ||  |{}{}|     |{}{}?/   \\{}{}?|     |{}?|     |" ),
+QLatin1String(
 "                                                          ____\n"
 "                                             __{}{}{}{}?/   o  \\\n"
 "             _{}{}{}{}{}{}{}{}{}{}?_{}{}{}?/    \\{}{}?/     ____ >\n"
 "   _{}{}{}?/   \\{}{}{}{}_{}{}{}{}/   \\{}{}|  __  |{}?|     |\n"
-"  | \\{}{}?|     |{}{}?/   \\{}{}?|     |{}?|  ||  |{}?|     |\n",
-
+"  | \\{}{}?|     |{}{}?/   \\{}{}?|     |{}?|  ||  |{}?|     |" ),
+QLatin1String(
 "                                                          ____\n"
 "                                  __{}{}{}{}{}{}{}{}{}{}/   o  \\\n"
 " _{}{}{}{}{}{}{}{}{}{}{}_{}{}{}?/    \\{}{}{}{}_{}{}{}?/     ____ >\n"
 "| \\{}{}{}{}{}_{}{}{}{}/   \\{}{}|  __  |{}{}?/   \\{}{}|     |\n"
-" \\ \\{}{}{}?/   \\{}{}?|     |{}?|  ||  |{}{}|     |{}?|     |",
-
+" \\ \\{}{}{}?/   \\{}{}?|     |{}?|  ||  |{}{}|     |{}?|     |" ),
+QLatin1String(
 "                                                          ____\n"
 "                       __{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}?/   o  \\\n"
 "  _{}{}{}{}{}_{}{}{}?/    \\{}{}{}{}_{}{}{}{}{}{}{}{}{}/     ____ >\n"
 " | \\{}{}{}?/   \\{}{}|  __  |{}{}?/   \\{}{}{}{}_{}{}{}|     |\n"
-"  \\ \\{}{}?|     |{}?|  ||  |{}{}|     |{}{}?/   \\{}{}|     |",
-
+"  \\ \\{}{}?|     |{}?|  ||  |{}{}|     |{}{}?/   \\{}{}|     |" ),
+QLatin1String(
 "    ____\n"
 "  /  o   \\{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}__\n"
 "< ____     \\{}{}{}?_{}{}{}{}{}{}{}{}{}{}?_{}{}{}{}/    \\\n"
 "      |     |{}{}/   \\{}{}{}{}_{}{}{}{}/   \\{}{}?|  __  |{}{}{}_\n"
-"      |     |{}?|     |{}{}?/   \\{}{}?|     |{}{}|  ||  |{}{}?/ |",
-
+"      |     |{}?|     |{}{}?/   \\{}{}?|     |{}{}|  ||  |{}{}?/ |" ),
+QLatin1String(
 "    ____\n"
 "  /  o   \\{}{}{}{}?__\n"
 "< ____     \\{}{}?/    \\{}{}{}?_{}{}{}{}{}{}{}{}{}{}?_\n"
 "      |     |{}?|  __  |{}{}/   \\{}{}{}{}_{}{}{}{}/   \\{}{}{}?_\n"
-"      |     |{}?|  ||  |{}?|     |{}{}?/   \\{}{}?|     |{}{}?/ |",
-
+"      |     |{}?|  ||  |{}?|     |{}{}?/   \\{}{}?|     |{}{}?/ |" ),
+QLatin1String(
 "    ____\n"
 "  /  o   \\{}{}{}{}{}{}{}{}{}{}__\n"
 "< ____     \\{}{}{}?_{}{}{}{}/    \\{}{}{}?_{}{}{}{}{}{}{}{}{}{}{}_\n"
 "      |     |{}{}/   \\{}{}?|  __  |{}{}/   \\{}{}{}{}_{}{}{}{}{}/ |\n"
-"      |     |{}?|     |{}{}|  ||  |{}?|     |{}{}?/   \\{}{}{}?/ /",
-
+"      |     |{}?|     |{}{}|  ||  |{}?|     |{}{}?/   \\{}{}{}?/ /" ),
+QLatin1String(
 "    ____\n"
 "  /  o   \\{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}?__\n"
 "< ____     \\{}{}{}{}{}{}{}{}{}_{}{}{}{}/    \\{}{}{}?_{}{}{}{}{}_\n"
 "      |     |{}{}{}_{}{}{}{}/   \\{}{}?|  __  |{}{}/   \\{}{}{}?/ |\n"
-"      |     |{}{}/   \\{}{}?|     |{}{}|  ||  |{}?|     |{}{}?/ /"
+"      |     |{}{}/   \\{}{}?|     |{}{}|  ||  |{}?|     |{}{}?/ /" )
     };
 
     QString nessie_mask[] = {
+QLatin1String(
 "\n"
 "                                                            W\n"
 "\n"
 "\n"
 "\n"
-"",
-
+"" ),
+QLatin1String(
 "\n"
 "     W\n"
 "\n"
 "\n"
 "\n"
-""
+"" )
     };
 
     int nessie_num = m_randomSequence.getLong(2); // 0 = right, 1 = left.
@@ -1235,7 +1342,7 @@ void AASaver::addNessie(Screen* screen)
 
 void AASaver::addRandom(Screen* screen)
 {
-    int choice = m_randomSequence.getLong(5);
+    int choice = m_randomSequence.getLong(6);
 
     switch(choice)
     {
@@ -1254,13 +1361,16 @@ void AASaver::addRandom(Screen* screen)
         case 4:
             addWhale(screen);
             break;
+        case 5:
+            addSubmarine(screen);
+            break;
     }
 }
 
 void AASaver::paintEvent(QPaintEvent* pe)
 {
     // Dirty region is passed instead of the entire viewport for efficiency.
-    screen->paint(pe->region());
+    screen->paint(pe);
 }
 
 //
@@ -1273,7 +1383,7 @@ AASaverInterface::~AASaverInterface()
 
 KAboutData *AASaverInterface::aboutData()
 {
-    return new KAboutData("kdeasciiquarium.kss", 0, ki18n("KDE Asciiquarium"), "0.4.1",
+    return new KAboutData("kdeasciiquarium.kss", "kdeasciiquarium", ki18n("KDE Asciiquarium"), "0.5.0",
         ki18n("KDE Asciiquarium"));
 }
 
@@ -1284,17 +1394,17 @@ KScreenSaver *AASaverInterface::create(WId id)
 
 QDialog *AASaverInterface::setup()
 {
-    KConfigDialog *dialog = KConfigDialog::exists("settings");
+    KConfigDialog *dialog = KConfigDialog::exists(QLatin1String("settings"));
     if(dialog)
         return dialog;
 
-    dialog = new KConfigDialog(0, "settings", AASaverConfig::self());
-
+    dialog = new KConfigDialog(0, QLatin1String("settings"), AASaverConfig::self());
+    KGlobal::locale()->insertCatalog(QLatin1String("kdeasciiquarium"));
     QWidget *settings = new QWidget(dialog);
     Ui::SettingsWidget ui;
     ui.setupUi(settings);
 
-    dialog->addPage(settings, i18n("Asciiquarium Settings"), "preferences-desktop-screensaver");
+    dialog->addPage(settings, i18n("Asciiquarium Settings"), QLatin1String("preferences-desktop-screensaver"));
 
     // Not much for help needed really...
     dialog->enableButton(KDialog::Help, false);
